#============================================================================#
# Dictionary that contains default keybinds and replacement names

# Keybinding: ['default command','description','user defined command']

# Note: Storing default command as string representation for comparison.
#       This may be removed in the future. Longer names are commented out and
#       truncated for this purpose.
#============================================================================#
shortcut_dict_ref = {
'left': ('_ backward',"previous movie frame",''),
'right': ('_ forward',"next movie frame",''),
'pgup': ('scene action=previous',"previous scene",''),
'pgdn': ('scene action=next',"last scene",''),
'home': ('zoom animate=-1',"zoom all",''),
'end': ('mtoggle','play/pause movie',''),
'insert': ('rock','',''),
'SHFT-left': ('backward','',''),
'SHFT-right': ('forward','',''),
'SHFT-pgup': ('scene action=previous','previous scene',''),
'SHFT-pgdn': ('scene action=next','next scene',''),
'SHFT-home': ('rewind','',''),
'SHFT-end': ('ending','',''),
'SHFT-insert': ('rock','',''),
'CTRL-left': ('backward','',''),
'CTRL-right': ('forward','',''),
'CTRL-pgup': ("_ scene new, insert_before",'insert scene before current',''),
'CTRL-pgdn': ("_ scene new, insert_after",'insert scene after current',''),
'CTRL-home': ('zoom animate=-1','zoom all',''),
'CTRL-end': ('scene new, store','store new scene',''),
'CTRL-insert': ('scene auto, store','store auto scene',''),
'CTRL-A': ('select sele, all, 1','select all',''),
'CTRL-C': ('editing_ring copy','copy',''),
'CTRL-F': ('wizard find','find',''),
'CTRL-H': ('help edit_keys','help',''),
'CTRL-I': ('editing_ring invert','invert selection',''),
'CTRL-L': ('util.ligand_zoom()','zoom next ligand',''),
'CTRL-T': ('bond;unpick','create bond',''),
'CTRL-V': ('editing_ring paste','paste',''),
'CTRL-X': ('editing_ring cut','cut',''),
'CTRL-Y': ('redo','',''),
'CTRL-Z': ('undo','',''),
'ALT-left': ('backward','',''),
'ALT-right': ('forward','',''),
'ALT-pgup': ('rewind','',''),
'ALT-pgdn': ('ending','',''),
'ALT-home': ('zoom animate=-1','zoom all',''),
'ALT-end': ('ending','',''),
'ALT-insert': ('rock','',''),
'ALT-1': ("editor.attach_fragment('pk1', 'formamide', 5, 0)",'attach amide N->C',''),
'ALT-2': ("editor.attach_fragment('pk1', 'formamide', 4, 0)",'attach amide C->N',''),
'ALT-3': ("editor.attach_fragment('pk1', 'sulfone', 3, 1)",'attach sulfone',''),
'ALT-4': ("editor.attach_fragment('pk1', 'cyclobutane', 4, 0)",'attach cyclobutane',''),
'ALT-5': ("editor.attach_fragment('pk1', 'cyclopentane', 5, 0)",'attach cyclopentane',''),
'ALT-6': ("editor.attach_fragment('pk1', 'cyclohexane', 7, 0)",'attach cyclohexane',''),
'ALT-7': ("editor.attach_fragment('pk1', 'cycloheptane', 8, 0)",'attach cycloheptane',''),
'ALT-8': ("editor.attach_fragment('pk1', 'cyclopentadiene', 5, 0)",'attach cyclopentadiene',''),
'ALT-9': ("editor.attach_fragment('pk1', 'benzene', 6, 0)",'attach benzene',''),
'ALT-0': ("editor.attach_fragment('pk1', 'formaldehyde', 2, 0)",'attach formaldehyde',''),
'ALT-A': ("editor.attach_amino_acid('pk1', 'ala')",'attach ala',''),
'ALT-B': ("editor.attach_amino_acid('pk1', 'ace')",'attach ace',''),
'ALT-C': ("editor.attach_amino_acid('pk1', 'cys')",'attach cys',''),
'ALT-D': ("editor.attach_amino_acid('pk1', 'asp')",'attach asp',''),
'ALT-E': ("editor.attach_amino_acid('pk1', 'glu')",'attach glu',''),
'ALT-F': ("editor.attach_amino_acid('pk1', 'phe')",'attach phe',''),
'ALT-G': ("editor.attach_amino_acid('pk1', 'gly')",'attach gly',''),
'ALT-H': ("editor.attach_amino_acid('pk1', 'his')",'attach his',''),
'ALT-I': ("editor.attach_amino_acid('pk1', 'ile')",'attach ile',''),
'ALT-J': ("editor.attach_fragment('pk1', 'acetylene', 2, 0)",'attach acetylene',''),
'ALT-K': ("editor.attach_amino_acid('pk1', 'lys')",'attach lys',''),
'ALT-L': ("editor.attach_amino_acid('pk1', 'leu')",'attach leu',''),
'ALT-M': ("editor.attach_amino_acid('pk1', 'met')",'attach met',''),
'ALT-N': ("editor.attach_amino_acid('pk1', 'asn')",'attach asn',''),
'ALT-P': ("editor.attach_amino_acid('pk1', 'pro')",'attach pro',''),
'ALT-Q': ("editor.attach_amino_acid('pk1', 'gln')",'attach gln',''),
'ALT-R': ("editor.attach_amino_acid('pk1', 'arg')",'attach arg',''),
'ALT-S': ("editor.attach_amino_acid('pk1', 'ser')",'attach ser',''),
'ALT-T': ("editor.attach_amino_acid('pk1', 'thr')",'attach thr',''),
'ALT-V': ("editor.attach_amino_acid('pk1', 'val')",'attach val',''),
'ALT-W': ("editor.attach_amino_acid('pk1', 'trp')",'attach trp',''),
'ALT-Y': ("editor.attach_amino_acid('pk1', 'tyr')",'attach tyr',''),
'ALT-Z': ("editor.attach_amino_acid('pk1', 'nme')",'attach nme',''),
'CTSH-left': ('backward','',''),
'CTSH-right': ('forward','',''),
'CTSH-pgup': ("scene new, insert_before",'insert scene before current',''),
'CTSH-pgdn': ("scene new, insert_after",'insert scene after current',''),
'CTSH-home': ('zoom animate=-1','zoom all',''),
'CTSH-end': ('mtoggle','',''),
'CTSH-insert': ('rock','',''),
'CTSH-A': ('redo','',''),
'CTSH-B': ('replace Br,1,1','',''),
'CTSH-C': ('replace C,4,4','',''),
'CTSH-D': ('remove_picked','',''),
'CTSH-E': ('invert','',''),
'CTSH-F': ('replace F,1,1','',''),
'CTSH-G': ('replace H,1,1','',''),
'CTSH-I': ('replace I,1,1','',''),
'CTSH-J': ('alter pk1,formal_charge=-1.','',''),
'CTSH-K': ('alter pk1,formal_charge=1.','',''),
'CTSH-L': ('replace Cl,1,1','',''),
'CTSH-N': ('replace N,4,3','',''),
'CTSH-O': ('replace O,4,2','',''),
'CTSH-P': ('replace P,4,1','',''),
'CTSH-R': ('h_fill','',''),
'CTSH-S': ('replace S,4,2','',''),
'CTSH-T': ('bond;unpick','',''),
'CTSH-U': ('alter pk1,formal_charge=0.','',''),
'CTSH-W': ('cycle_valence','',''),
'CTSH-X': ("cmd.auto_measure()",'auto measure',''),
'CTSH-Y': ('attach H,1,1','',''),
'CTSH-Z': ('undo','',''),
'CTRL-F1': ("scene F1, store",'',''),
'CTSH-F1': ("scene SHFT-F1, store",'',''),
'CTRL-F2': ("scene F2, store",'',''),
'CTSH-F2': ("scene SHFT-F2, store",'',''),
'CTRL-F3': ("scene F3, store",'',''),
'CTSH-F3': ("scene SHFT-F3, store",'',''),
'CTRL-F4': ("scene F4, store",'',''),
'CTSH-F4': ("scene SHFT-F4, store",'',''),
'CTRL-F5': ("scene F5, store",'',''),
'CTSH-F5': ("scene SHFT-F5, store",'',''),
'CTRL-F6': ("scene F6, store",'',''),
'CTSH-F6': ("scene SHFT-F6, store",'',''),
'CTRL-F7': ("scene F7, store",'',''),
'CTSH-F7': ("scene SHFT-F7, store",'',''),
'CTRL-F8': ("scene F8, store",'',''),
'CTSH-F8': ("scene SHFT-F8, store",'',''),
'CTRL-F9': ("scene F9, store",'',''),
'CTSH-F9': ("scene SHFT-F9, store",'',''),
'CTRL-F10': ("scene F10, store",'',''),
'CTSH-F10': ("scene SHFT-F10, store",'',''),
'CTRL-F11': ("scene F11, store",'',''),
'CTSH-F11': ("scene SHFT-F11, store",'',''),
'CTRL-F12': ("scene F12, store",'',''),
'CTSH-F12': ("scene SHFT-F12, store",'',''),
}
#============================================================================#



'''
Derived from freemol.mpeg_encode

Copyright (c) Schrodinger, Inc.
'''


def validate():
    '''
    Return the path to the "mpeg_encode" executable or None if not found
    '''
    from shutil import which
    return which('mpeg_encode') or which('mpeg_encode.exe')


INPUT_TEMPLATE = """
# parameter file template with lots of comments to assist you
#
# you can use this as a template, copying it to a separate file then modifying
# the copy
#
#
# any line beginning with '#' is a comment
#
# no line should be longer than 255 characters
#
#
# general format of each line is:
#	<option> <spaces and/or tabs> <value>
#
# lines can generally be in any order
#
# an exception is the option 'INPUT' which must be followed by input
# files in the order in which they must appear, followed by 'END_INPUT'
#
# Also, if you use the `command` method of generating input file names,
# the command will only be executed in the INPUT_DIR if INPUT_DIR preceeds
# the INPUT parameter.
#
# <option> MUST be in UPPER CASE
#

#PATTERN		IPBBBPBBBB
#PATTERN		IPBBPBBPBB
#PATTERN		IPBPBPBPBP
PATTERN		    IPPPPPPPPP

FORCE_ENCODE_LAST_FRAME

OUTPUT		%s

# mpeg_encode really only accepts 3 different file formats, but using a
# conversion statement it can effectively handle ANY file format
#
# You must specify the type of the input files.  The choices are:
#    YUV, PPM, JMOVIE, Y, JPEG, PNM
#	(must be upper case)
#
BASE_FILE_FORMAT	PPM

#
# if YUV format (or using parallel version), must provide width and height
# YUV_SIZE	widthxheight
# this option is ignored if BASE_FILE_FORMAT is not YUV and you're running
# on just one machine
#
# YUV_SIZE	352x240

# If you are using YUV, there are different supported file formats.
# EYUV or UCB are the same as previous versions of this encoder.
# (All the Y's, then U's then V's, in 4:2:0 subsampling.)
# Other formats, such as Abekas, Phillips, or a general format are
# permissible, the general format is a string of Y's, U's, and V's
# to specify the file order.

#INPUT_FORMAT UCB

# the conversion statement
#
# Each occurrence of '*' will be replaced by the input file
#
# e.g., if you have a bunch of GIF files, then this might be:
#	INPUT_CONVERT	giftoppm *
#
# e.g., if you have a bunch of files like a.Y a.U a.V, etc., then:
#	INPUT_CONVERT	cat *.Y *.U *.V
#
# e.g., if you are grabbing from laser disc you might have something like
#	INPUT_CONVERT	goto frame *; grabppm
# 'INPUT_CONVERT *' means the files are already in the base file format
#
INPUT_CONVERT	*

# number of frames in a GOP.
#
# since each GOP must have at least one I-frame, the encoder will find the
# the first I-frame after GOP_SIZE frames to start the next GOP
#
# later, will add more flexible GOP signalling
#
GOP_SIZE	10

# number of slices in a frame
#
# 1 is a good number.  another possibility is the number of macroblock rows
# (which is the height divided by 16)
#
SLICES_PER_FRAME	1

# directory to get all input files from (makes this file easier to read)
INPUT_DIR 	%s

# There are a bunch of ways to specify the input files.
# from a simple one-per-line listing, to the following 
# way of numbering them.  See the manual for more information.
INPUT
# '*' is replaced by the numbers 01, 02, 03, 04
# if I instead do [01-11], it would be 01, 02, ..., 09, 10, 11
# if I instead do [1-11], it would be 1, 2, 3, ..., 9, 10, 11
# if I instead do [1-11+3], it would be 1, 4, 7, 10
# the program assumes none of your input files has a name ending in ']'
# if you do, too bad!!!
#
#
%s*.ppm 	[%04d-%04d]

# can have more files here if you want...there is no limit on the number
# of files
END_INPUT

# Many of the remaining options have to do with the motion search and qscale

# FULL or HALF -- must be upper case
PIXEL		HALF

# means +/- this many pixels for both P and B frame searches
# specify two numbers if you wish to serc different ranges in the two.
RANGE		16

# this must be one of {EXHAUSTIVE, SUBSAMPLE, LOGARITHMIC}
PSEARCH_ALG	LOGARITHMIC

# this must be one of {SIMPLE, CROSS2, EXHAUSTIVE}
#
# note that EXHAUSTIVE is really, really, really slow
#
BSEARCH_ALG	CROSS2

#
# these specify the q-scale for I, P, and B frames
# (values must be between 1 and 31)
# These are the Qscale values for the entire frame in variable bit-rate
# mode, and starting points (but not important) for constant bit rate
#
IQSCALE		%d
PQSCALE		%d
BQSCALE		%d

# this must be ORIGINAL or DECODED
REFERENCE_FRAME	DECODED

# for parallel parameters see parallel.param in the exmaples subdirectory

# if you want constant bit-rate mode, specify it as follows (number is bits/sec):
#BIT_RATE  8000000

# To specify the buffer size (327680 is default, measused in bits, for 16bit words)
#BUFFER_SIZE 327680

# The frame rate is the number of frames/second (legal values:
# 23.976, 24, 25, 29.97, 30, 50 ,59.94, 60
FRAME_RATE 30

# There are many more options, see the users manual for examples....
# ASPECT_RATIO, USER_DATA, GAMMA, IQTABLE, etc.


"""


def input(filename, path, prefix, first, last, quality=12):
    '''
    Return a parameter file (file contents).
    '''
    return INPUT_TEMPLATE % (
        filename,
        path,
        prefix,
        first,
        last,
        min(quality, 31),
        min(quality + 1, 31),
        min(quality + 2, 31),
    )


def run(paraminput):
    '''
    Run "mpeg_encode"

    :param paraminput: Parameter file contents
    :return: (stdout, stderr) bytes tuple
    '''
    from subprocess import Popen, PIPE
    mpeg_encode_exe = validate()

    if not mpeg_encode_exe:
        raise FileNotFoundError('Cannot find "mpeg_encode"')

    if not isinstance(paraminput, bytes):
        paraminput = paraminput.encode()

    p = Popen([mpeg_encode_exe], stdin=PIPE, stdout=PIPE, stderr=PIPE)

    return p.communicate(paraminput)
